% This Matlab code illustrates the proposed estimators DWWS and DWWS-LPM 
% on the inductance plethysmography data set (IPD). Note that c=1/5 is
% used.
%
%  The program calls to:
%           bayesrule_dwws_trans.m -- Computes the posterior mean for the 
%                                     DWWS estimator by numerical 
%                                     integration using a transformation to
%                                     remove singularity. Any 0<c<1 can be 
%                                     used.
%           bayesrule_dwwslpm.m -- Computes the larger posterior mode (LPM)
%                                  for the DWWS-LPM estimator. Note that 
%                                  any rational 0<c<1 in the form of 
%                                  c=cm/cn can be used.
%--------------------------------------------------------------------------
clear all;
close all;
set(0, 'DefaultAxesFontSize', 16);

load ipd.dat;  % IPD data set
N = length(ipd);
coarsest = floor(log2(log(N))) + 1;  % coarsest level of decomposition
finest_lev = log2(N)-1;  % finest level
filt = MakeONFilter('Symmlet', 8);  % wavelet filter
sn = ipd';

data_wd = FWT_PO (sn, coarsest, filt);  % wavelet transform
data_smowd1 = data_wd;
data_smowd2 = data_wd;

niz = data_wd(dyad(finest_lev));
sigma_hat = mad(niz,1)/0.6745;  % estimate noise
    
c=1/5;  
cm = 1;
cn = 5;

for i = coarsest:finest_lev
    a = data_wd(dyad(i));
    varr = max(0,var(a) - sigma_hat^2);
    b(i) = (varr/gamma(1+2/c))^(c/2);  % set parameter b levelwise
    a_out1 = 0 .* a; 
    a_out2 = 0 .* a;
    nn=length(a);
    for  j = 1:nn
        a_out1(j) = bayesrule_dwws_trans(a(j), sigma_hat^2, c, b(i));
        a_out2(j) = bayesrule_dwwslpm(a(j), sigma_hat^2, cm, cn, b(i));
    end;
    data_smowd1(dyad(i)) = a_out1; 
    data_smowd2(dyad(i)) = a_out2;
end;
    
smo_data1 = IWT_PO( data_smowd1, coarsest, filt);  % inverse wavelet transfrom
smo_data2 = IWT_PO( data_smowd2, coarsest, filt);

figure; plot((1:N)/N,sn,'LineWidth',1.0);
title('Inductance plethysmography data');

figure; plot((1:N)/N,smo_data1,'LineWidth',1.0);
title('DWWS reconstruction'); ylim([-0.2 1.2]);

figure; plot((1:N)/N,smo_data2,'LineWidth',1.0); 
title('DWWS-LPM reconstruction'); ylim([-0.2 1.2]);
